#ifndef _sw_salt_h
#define _sw_salt_h

/*
 * Copyright 2003 Swampwolf Technologies, Inc. All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification,
 *	are permitted provided that the following conditions are met:
 *
 *		1. Redistributions of source code must retain the above copyright notice,
 *		   this list of conditions and the following disclaimer.   
 *		2. Redistributions in binary form must reproduce the above copyright notice,
 *		   this list of conditions and the following disclaimer in the documentation
 *		   and/or other materials provided with the distribution.
 *
 *	THIS SOFTWARE IS PROVIDED BY SWAMPWOLF TECHNOLOGIES, INC. ``AS IS'' AND ANY
 *	EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *	IN NO EVENT SHALL THE HOWL PROJECT OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *	BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 *	OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 *	OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 *	OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *	The views and conclusions contained in the software and documentation are those
 *	of the authors and should not be interpreted as representing official policies,
 *	either expressed or implied, of Swampwolf Technologies, Inc.
 */

#include <salt/vtypes.h>
#include <salt/time.h>
#include <salt/dll.h>


#ifdef __cplusplus
extern "C"
{
#endif 


typedef enum _sw_socket_event
{
   SW_SOCKET_READ		=	0x1,
   SW_SOCKET_WRITE	=	0x2,
   SW_SOCKET_OOB		=	0x4
} sw_socket_event;


struct							_sw_salt;
typedef struct _sw_salt	*	sw_salt;
struct							_sw_socket;
struct							_sw_timer;
struct							_sw_network_interface;
struct							_sw_signal;

typedef sw_opaque				sw_socket_handler;
typedef sw_result 			(*sw_socket_handler_func)(
													sw_socket_handler		handler,
													sw_salt					salt,
													struct _sw_socket	*	socket,
													sw_socket_event		events,
													sw_opaque				extra);

typedef sw_opaque				sw_timer_handler;
typedef sw_result				(*sw_timer_handler_func)(
													sw_timer_handler		handler,
													sw_salt					salt,
													struct _sw_timer	*	timer,
													sw_time					timeout,
													sw_opaque				extra);

typedef sw_opaque				sw_network_interface_handler;
typedef sw_result				(*sw_network_interface_handler_func)(
													sw_network_interface_handler		handler,
													sw_salt									salt,
													struct _sw_network_interface	*	netif,
													sw_opaque								extra);

typedef sw_opaque				sw_signal_handler;
typedef sw_result				(*sw_signal_handler_func)(
													sw_signal_handler		handler,
													sw_salt					salt,
													struct _sw_signal	*	signal,
													sw_opaque				extra);


sw_result SALT_DLL
sw_salt_init(
				sw_salt		*	self,
				int				argc,
				char			**	argv);
                                 
                                 
sw_result SALT_DLL
sw_salt_fina(
				sw_salt	self);

                                                                  
sw_result SALT_DLL
sw_salt_register_socket(
				sw_salt						self,
				struct _sw_socket		*	socket,
				sw_socket_event			events,
				sw_socket_handler			handler,
				sw_socket_handler_func	func,
				sw_opaque					extra);


sw_result SALT_DLL
sw_salt_unregister_socket(
				sw_salt						self,
				struct _sw_socket		*	socket);
                                 
                                 
sw_result SALT_DLL
sw_salt_register_timer(
				sw_salt						self,
				struct _sw_timer		*	timer,
				sw_time						timeout,
				sw_timer_handler			handler,
				sw_timer_handler_func	func,
				sw_opaque					extra);
                                 
                                 
sw_result SALT_DLL
sw_salt_unregister_timer(
				sw_salt						self,
				struct _sw_timer		*	timer);


sw_result SALT_DLL
sw_salt_register_network_interface(
				sw_salt										self,
				struct _sw_network_interface		*	netif,
				sw_network_interface_handler			handler,
				sw_network_interface_handler_func	func,
				sw_opaque									extra);


sw_result SALT_DLL
sw_salt_unregister_network_interface_handler(
				sw_salt						self);


sw_result SALT_DLL
sw_salt_register_signal(
				sw_salt						self,
				struct _sw_signal	*		signal,
				sw_signal_handler			handler,
				sw_signal_handler_func	func,
				sw_opaque					extra);


sw_result SALT_DLL
sw_salt_unregister_signal(
				sw_salt						self,
				struct _sw_signal	*		signal);


sw_result SALT_DLL
sw_salt_lock(
				sw_salt	self);


sw_result SALT_DLL
sw_salt_unlock(
				sw_salt	self);


sw_result SALT_DLL
sw_salt_step(
				sw_salt		self,
				sw_ulong	*	msec);


sw_result SALT_DLL
sw_salt_run(
				sw_salt	self);


sw_result SALT_DLL
sw_salt_stop_run(
				sw_salt	self);
            

#ifdef __cplusplus
}
#endif


#endif
